// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition" by Clifford A. Shaffer, Prentice Hall, 2007.
// Source code Copyright (C) 2006 by Clifford A. Shaffer.

// Test program for various preorder traversals of a binary tree.
// This includes two implementations for preorder traversal,
// and a traversal that counts the number of nodes in the tree.
#include <iostream>
using namespace std;

#include "book.h"

#include "binnode.ht"

// Stick these in to test the preorder routine
#define visit(X) (cout << X->val() << endl)

// This is the preferred structure for preorder traversal.
// It tests the pointer to the current node before visiting it,
// but does not test that children are NULL before calling a
// visit on them. This version does not rely on the caller
// to test the root for safety.
template <class Elem>
void preorder(BinNode<Elem>* root) {
  if (root == NULL) return; // Empty subtree, do nothing
  visit(root);              // Perform desired action
  preorder(root->left());
  preorder(root->right());
}

// Count the number of nodes in a binary tree
template <class Elem>
int count(BinNode<Elem>* root) {
  if (root == NULL) return 0;  // Nothing to count
  return 1 + count(root->left())
           + count(root->right());
}

// This version of preorder does "look ahead" to test if
// a child should be visited before doing the visit.
// This requires an external test of the root to make it safe
// from calls on an empty tree.
template <class Elem>
void preorder2(BinNode<Elem>* root) {
  visit(root);  // Perform whatever action is desired
  if (root->left() != NULL) preorder2(root->left());
  if (root->right() != NULL) preorder2(root->right());
}

int main()
{
  BinNodePtr<Int>* root = new BinNodePtr<Int>(10);
  root->setLeft(new BinNodePtr<Int>(15));
  root->setRight(new BinNodePtr<Int>(20));
  preorder(root);
  cout << " Count is: " << count(root) << endl;
  preorder2(root);
}
